#include "stdafx.h"
#include "ModelManager.h"
#include "WorkitemSerializer.h"
#include "DefWorkitem.h"
#include "Maquina.h"
#include "Rol.h"
#include "Configuracion.h"
#include "CompressorWrp.h"
#include <BXMLWrapper.h>
#include <ParseXml.h>
#include "Globales.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
CModelManager::CModelManager() {
	g_objConfig = new CConfiguracion;
	g_objConfig->Initialize();
}

CModelManager::~CModelManager() {
	DELETE_NOT_NULL(g_objConfig);
	m_mapaRoles.Destroy();
}

//////////////////////////////////////////////////////////////////////
// Carga de un XML un workitem, segn el rol al que est asociado,
// y su status (template o redefinido)
bool CModelManager::LoadFromXML(LPCTSTR p_strFilename, CDefWorkitem** p_objWki) {
	CParseXML objParser;
	CWorkitemSerializer objSerializer;
	CDefWorkitem* objWki;

	if(!objParser.Load(p_strFilename)) {
		return false;
	}
	CXMLNode* objNode = objParser.GetNodo(_T("//root/Def_Workitems/*"));
	if(objSerializer.FromXml(&objWki, objNode)) {
		*p_objWki = objWki;
		delete objNode;
		return true;
	}
	return false;
}

//////////////////////////////////////////////////////////////////////
// Carga de la base un workitem, segn el rol al que est asociado,
// y su status (template o redefinido)
ModelLoadResult CModelManager::LoadFromBD(LPCTSTR p_strWkID, LPCTSTR p_strRol, CDefWorkitem** p_objWki) {
	CParseXML objParser;
	CWorkitemSerializer objSerializer;
	CADODatabase pAdoDb;
	CString strSql, strCodigo;

	pAdoDb.SetConnectionString(g_objConfig->GetSQLConnectionString());
	if(!pAdoDb.Open()) {
		return FAIL_DATABASE_ERROR;
	}
	strSql.Format(
		_T("exec ObtCfgAppComp '%s', '%s', '%s', 'P', '%s', 'RETRIEVE'"),
		g_objConfig->GetAppName(), g_objConfig->GetAppVer(), p_strRol, p_strWkID);
	CADORecordset pAdoRs(&pAdoDb);
	pAdoRs.Open(strSql, CADORecordset::openQuery);
	if(pAdoRs.GetRecordCount()) {
		strCodigo = _T("");
		pAdoRs.MoveFirst();
		while(!pAdoRs.IsEOF()) {
			_variant_t vtValue;
			for(int i = 1; i < pAdoRs.GetFieldCount(); i++) {
				pAdoRs.GetFieldValue(i, vtValue);
				strCodigo += (LPCTSTR)(_bstr_t)vtValue;
			}
			pAdoRs.MoveNext();
		}
		pAdoRs.Close();
	}
	pAdoDb.Close();

	CDefWorkitem* objWki;
	if(strCodigo == _T("")) {
		// No hay cdigo en la base
		objWki = new CDefWorkitem;
		objWki->SetMaquina(new CMaquina);
		objWki->SetTipo(p_strWkID);
		*p_objWki = objWki;
		return WARNING_EMPTY_WORKITEM;
	}

	// Ya tengo el cdigo, ahora descomprimir y levantar el cdigo
	CCompressorWrp objComp;

	if(!objComp.Descomprimir(strCodigo)) {
		return FAIL_CANT_UNCOMPRESS;
	}
	if(!objParser.LoadXML(strCodigo)) {
		return FAIL_WRONG_XML;
	}
	CXMLNode* objNode = objParser.GetNodo(_T("//root/Def_Workitems/*"));
	if(objSerializer.FromXml(&objWki, objNode)) {
		*p_objWki = objWki;
		delete objNode;
		return RES_OK;
	}
	return FAIL_WRONG_XML;
}

//////////////////////////////////////////////////////////////////////
// 
bool CModelManager::SaveIntoBD(LPCTSTR p_strRol, CDefWorkitem* p_objWki) {
	CADODatabase pAdoDb;
	CString strSql, strRet;
	CWorkitemSerializer objSerializer;
	
	pAdoDb.SetConnectionString(g_objConfig->GetSQLConnectionString());
	if(!pAdoDb.Open()) {
		return false;
	}
	CString strOut;
	strOut = _T("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	strOut += _T("<root><Def_Workitems>");
	BOOL bRet = objSerializer.ToXml(p_objWki, strOut);
	strOut += _T("</Def_Workitems></root>");
	if(!bRet) {
		return false;
	}
	CCompressorWrp objComp;
	if(!objComp.ComprimirB64(strOut, strRet)) {
		return false;
	}

	pAdoDb.BeginTransaction();
	strSql.Format(
		_T("exec UpdCfgAppComp '%s', '%s', '%s', 'P', '%s', '%s', 'T'"),
		g_objConfig->GetAppName(), g_objConfig->GetAppVer(), p_strRol, 
		p_objWki->GetTipo(), strRet);
	if(pAdoDb.Execute(strSql)) {
		pAdoDb.CommitTransaction();
		return true;
	}
	pAdoDb.RollbackTransaction();
	return false;
}

//////////////////////////////////////////////////////////////////////
// 
bool CModelManager::AssociateWithTemplate(LPCTSTR p_strRol, LPCTSTR p_strWkiTipo) {
	CADODatabase pAdoDb;
	CString strSql;
	CWorkitemSerializer objSerializer;
	
	pAdoDb.SetConnectionString(g_objConfig->GetSQLConnectionString());
	if(!pAdoDb.Open()) {
		return false;
	}
	pAdoDb.BeginTransaction();
	strSql.Format(
		_T("exec DelCfgAppComp '%s', '%s', '%s', 'P', '%s', 'R'"),
		g_objConfig->GetAppName(), g_objConfig->GetAppVer(), p_strRol, p_strWkiTipo);
	if(pAdoDb.Execute(strSql)) {
		pAdoDb.CommitTransaction();
		return true;
	}
	pAdoDb.RollbackTransaction();
	return false;
}

//////////////////////////////////////////////////////////////////////
// 
bool CModelManager::SaveIntoXML(LPCTSTR p_strFilename, CDefWorkitem* p_objWki) {
	CWorkitemSerializer objSerializer;
	BOOL bRet;
	CString strOut;

	strOut = _T("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	strOut += _T("<root><Def_Workitems>");
	bRet = objSerializer.ToXml(p_objWki, strOut);
	strOut += _T("</Def_Workitems></root>");

	if(!bRet) {
		return false;
	}
	CFile file;
	file.Open(p_strFilename, CFile::modeCreate | CFile::modeWrite);
	file.Write(strOut, strOut.GetLength());
	file.Close();
	return bRet;
}

//////////////////////////////////////////////////////////////////////
// 
bool CModelManager::SaveIntoSQL(LPCTSTR p_strFilename, LPCTSTR p_strWkID, LPCTSTR p_strRol, CDefWorkitem* p_objWki) {
	CString strSql, strRet;
	CWorkitemSerializer objSerializer;
	bool bRet;
	CString strOut;

	strOut = _T("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	strOut += _T("<root><Def_Workitems>");
	bRet = objSerializer.ToXml(p_objWki, strOut);
	strOut += _T("</Def_Workitems></root>");

	if(!bRet) {
		return false;
	}
	
	CCompressorWrp objComp;
	if(!objComp.ComprimirB64(strOut, strRet)) {
		return false;
	}
	strSql.Format(
		_T("exec UpdCfgAppComp '%s', '%s', '%s', 'P', '%s', '%s', 'T'"),
		g_objConfig->GetAppName(), g_objConfig->GetAppVer(), p_strRol,
		p_strWkID, strRet);
	CFile file;
	file.Open(p_strFilename, CFile::modeCreate | CFile::modeWrite);
	file.Write(strSql, strSql.GetLength());
	file.Close();
	return bRet;
}

//////////////////////////////////////////////////////////////////////
// 
bool CModelManager::LoadRoles() {
	CADODatabase pAdoDb;
	CString strSql;
	DWORD iRaff;

	pAdoDb.SetConnectionString(g_objConfig->GetSQLConnectionString());
	if(!pAdoDb.Open()) {
		return false;
	}
	CADORecordset pAdoRs(&pAdoDb);
	CADORecordset pAdoRsWrk(&pAdoDb);
	CADORecordset pAdoRsWrk2(&pAdoDb);
	if(!pAdoRs.Open(
		_T("select distinct IdUsua0218, DesUsu0218 from PartisAs0222, LoggUsua0218 where Grants0222 = 'TEMPLATE' and TipObj0222='W' and IdUsua0218 = F1Usua02180222"),
		CADORecordset::openQuery)) {
		pAdoRs.Close();
		pAdoDb.Close();
		return false;
	}
	iRaff = pAdoRs.GetRecordCount();
	if(iRaff == 0) {
		LogInfo(_T("No hay roles"));
		pAdoRs.Close();
		pAdoDb.Close();
		return false;
	}
	
	pAdoRs.MoveFirst();
	while(!pAdoRs.IsEOF()) {
		CRol* objRol = new CRol;
		_variant_t vtValue;
		pAdoRs.GetFieldValue(_T("IdUsua0218"), vtValue);
		objRol->SetID((_bstr_t)vtValue);
		pAdoRs.GetFieldValue(_T("DesUsu0218"), vtValue);
		objRol->SetDescripcion((_bstr_t)vtValue);

		strSql.Format(_T("exec GetWrkPerfil '%s'"), objRol->GetID());
		pAdoRsWrk.Open(strSql, CADORecordset::openQuery);
		if(pAdoRsWrk.GetRecordCount()) {
			pAdoRsWrk.MoveFirst();
			while(!pAdoRsWrk.IsEOF()) {
				_variant_t vtValueID, vtValueVersion, vtValueStatus;;
				pAdoRsWrk.GetFieldValue(_T("IdObje0222"), vtValueID);
				pAdoRsWrk.GetFieldValue(_T("Versio0222"), vtValueVersion);
				strSql.Format(_T("exec ObtCfgAppComp '%s', '%s', '%s', 'P', '%s', 'QUERY'"),
					g_objConfig->GetAppName(), g_objConfig->GetAppVer(), objRol->GetID(), (LPCTSTR)(_bstr_t)vtValueID);
				pAdoRsWrk2.Open(strSql, CADORecordset::openQuery);
				if(pAdoRsWrk2.GetRecordCount()) {
					pAdoRsWrk2.MoveFirst();
					while(!pAdoRsWrk2.IsEOF()) {
						objRol->AddWorkitem((_bstr_t)vtValueID, (_bstr_t)vtValueVersion);
						pAdoRsWrk2.MoveNext();
					}
					pAdoRsWrk2.Close();
				}					
				pAdoRsWrk.MoveNext();
			}
			pAdoRsWrk.Close();
		}
		m_mapaRoles.SetAt(objRol->GetID(), objRol);
		pAdoRs.MoveNext();
	}
	pAdoRs.Close();
	pAdoDb.Close();
	return true;
}

//////////////////////////////////////////////////////////////////////
// 
CMyMapIterador<CRol*>* CModelManager::GetRoles() {
	return new CMyMapIterador<CRol*>(&m_mapaRoles);
}