#include "stdafx.h"
#include "WorkitemSerializer.h"
#include "DefWorkitem.h"
#include "Maquina.h"
#include "Estado.h"
#include "Transicion.h"
#include "TimedAction.h"
#include "Metodo.h"
#include "EventosMW.h"
#include "EventoMW.h"
#include "Iuff.h"
#include <BXMLWrapper.h>
#include <ParseXml.h>
#include <XmlNode.h>
#include <XmlAttribute.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
CWorkitemSerializer::CWorkitemSerializer() {
	
}

CWorkitemSerializer::~CWorkitemSerializer() {

}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::ToXml(CDefWorkitem* p_objWki, CString& strOut) {
	m_objWorkitem = p_objWki;

	strOut += _T("<Workitem tipo=\"");
	strOut += m_objWorkitem->GetTipo();
	strOut += _T("\"><Descripcion>");
	strOut += m_objWorkitem->GetDescripcion();
	strOut += _T("</Descripcion><Canal>");
	strOut += m_objWorkitem->GetCanalFisico();
	strOut += _T("</Canal>");
	MaquinaToXml(strOut);

	CMyMapIterador<CMetodo*>* iterador =  m_objWorkitem->GetMetodos();
	if(iterador->begin()) {
		strOut += _T("<Metodos>");
		while(*iterador) {
			CMetodo* oMet = (*iterador)++;
			MetodoToXml(oMet, strOut);
		}
		strOut += _T("</Metodos>");
	}
	delete iterador;
	EventosMWToXml(strOut);
	strOut += _T("</Workitem>");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::FromXml(CDefWorkitem** p_objWki, CXMLNode* node) {
	m_objWorkitem = new CDefWorkitem;
	CXMLNode* oNodo;
	m_objWorkitem->SetTipo(node->GetAttributeOfName(_T("tipo"))->GetValue());
	m_objWorkitem->SetDescripcion(node->GetChildOfName(_T("Descripcion"))->GetValue());
	m_objWorkitem->SetCanal(node->GetChildOfName(_T("Canal"))->GetValue());

	oNodo = node->GetChildOfName(_T("Maquina"));
	MaquinaFromXml(oNodo);

	oNodo = node->GetChildOfName(_T("Metodos"));
	for(int i = 0; i < oNodo->GetChildCount(); i++) {
		CXMLNode* oMet = oNodo->GetChildAt(i);
		MetodoFromXml(oMet);		
	}

	oNodo = node->GetChildOfName(_T("EventosMW"));
	EventosMWFromXml(oNodo);
	
	*p_objWki = m_objWorkitem;
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::MaquinaFromXml(CXMLNode* node) {
	CMaquina* objMaquina;
	CXMLNode* nodEstados	= node->GetChildOfName(_T("Estados"));
	CXMLNode* nodTrans		= node->GetChildOfName(_T("Transiciones"));
	int i;

	objMaquina = new CMaquina;
	m_objWorkitem->SetMaquina(objMaquina);
	for(i = 0; i < nodEstados->GetChildCount(); i++) {
		CXMLNode* nodEst = nodEstados->GetChildAt(i);
		EstadoFromXml(nodEst);
	}

	for(i = 0; i < nodTrans->GetChildCount(); i++) {
		CXMLNode* nodTrn = nodTrans->GetChildAt(i);
		TransicionFromXml(nodTrn);
	}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::EstadoFromXml(CXMLNode* node) {
	CEstado* objEst = new CEstado;

	objEst->SetID(node->GetAttributeOfName(_T("id"))->GetValue());
	// Posicion y tamao
	CRect rc; long lLeft, lTop, lWidth, lHeight;
	lLeft = _ttoi(node->GetAttributeOfName(_T("Left"))->GetValue());
	lTop = _ttoi(node->GetAttributeOfName(_T("Top"))->GetValue());
	lWidth = _ttoi(node->GetAttributeOfName(_T("Width"))->GetValue());
	lHeight = _ttoi(node->GetAttributeOfName(_T("Height"))->GetValue());
	rc.SetRect(lLeft, lTop, lLeft + lWidth, lTop + lHeight);
	objEst->SetRect(rc);

	// Descripcion
	CXMLNode* nod;
	nod = node->GetChildOfName(_T("Descripcion"));
	if(nod) {
		objEst->SetDescripcion(nod->GetValue());
	}
	nod = node->GetChildOfName(_T("Initial"));
	if(nod) {
		objEst->SetInitial(TRUE);
	}
	nod = node->GetChildOfName(_T("OnEntry"));
	if(nod) {
		objEst->SetOnEntryFromXml(nod->GetValue());
	}
	nod = node->GetChildOfName(_T("OnExit"));
	if(nod) {
		objEst->SetOnExitFromXml(nod->GetValue());
	}
	nod = node->GetChildOfName(_T("TimedActions"));
	if(nod) {
		for(int i = 0; i < nod->GetChildCount(); i++) {
			CXMLNode* nodTA = nod->GetChildAt(i);
			CTimedAction* objTA = new CTimedAction;

			LPCTSTR strTime = nodTA->GetAttributeOfName("time")->GetValue();
			if(lstrcmp(strTime, "evaluate") != 0) {
				objTA->SetTime(strTime);
				objTA->SetCodigo(nodTA->GetValue());
			}
			else {
				CXMLNode* objNodoCodeTime = nodTA->GetChildOfName("CodeTime");
				CXMLNode* objNodoCode = nodTA->GetChildOfName("Code");
				LPCTSTR strPrueba = nodTA->GetValue();
				LPCTSTR strPrueba2 = objNodoCodeTime->GetValue();
				objTA->SetTimeNeedsEval();
				objTA->SetTime(objNodoCodeTime->GetValue());
				objTA->SetCodigo(objNodoCode->GetValue());
			}
			objEst->AddTimedAction(objTA);
		}
	}
	m_objWorkitem->GetMaquina()->AddEstado(objEst);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::TransicionFromXml(CXMLNode* node) {
	CTransicion* objTran = new CTransicion;
	CEstado *estOrigen, *estDestino;

	LPCTSTR strFrom = node->GetAttributeOfName(_T("from"))->GetValue();
	LPCTSTR strTo = node->GetAttributeOfName(_T("to"))->GetValue();
	LPCTSTR strNombre = node->GetAttributeOfName(_T("accion"))->GetValue();
	estOrigen = m_objWorkitem->GetMaquina()->GetEstado(strFrom);
	estDestino = m_objWorkitem->GetMaquina()->GetEstado(strTo);
	if(!(estOrigen && estDestino)) {
		LogError(_T("CWorkitemSerializer::TransicionFromXml - Transicin invlida desde %s a %s"), strFrom, strTo);
		LogError(_T("CWorkitemSerializer::TransicionFromXml - Uno de los estados no existe"));
		delete objTran;
		return FALSE;
	}
	objTran->SetOrigen(estOrigen);
	objTran->SetDestino(estDestino);
	objTran->SetNombre(strNombre);
	CXMLNode* nod = node->GetChildOfName(_T("Accion"));
	if(nod) {
		objTran->SetAccion(nod->GetValue());
		objTran->SetTipoAccion(nod->GetAttributeOfName(_T("type"))->GetValue());
	}
	nod = node->GetChildOfName(_T("Descripcion"));
	if(nod) {
		objTran->SetDescripcion(nod->GetValue());
	}
	nod = node->GetChildOfName(_T("Condicion"));
	if(nod) {
		objTran->SetCondicion(nod->GetValue());
	}
	nod = node->GetChildOfName(_T("ExtraPoints"));
	if(nod) {
		for(int i = 0; i < nod->GetChildCount(); i++) {
			CXMLNode* nodPt = nod->GetChildAt(i);
			CPoint pt(
				_ttol(nodPt->GetAttributeOfName(_T("x"))->GetValue()),
				_ttol(nodPt->GetAttributeOfName(_T("y"))->GetValue())
				);
			objTran->AddExtraPoint(pt);
		}
	}
	estOrigen->AddTransicion(objTran);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::MetodoFromXml(CXMLNode* node) {
	CMetodo* objMet = new CMetodo;

	objMet->SetNombre(node->GetAttributeOfName(_T("nombre"))->GetValue());
	objMet->SetCodigo(node->GetValue());

	m_objWorkitem->AddMetodo(objMet);

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::EventosMWFromXml(CXMLNode* node) {
	CEventosMW& objEvts = m_objWorkitem->GetEventosMW();

	objEvts.SetPeticion(node->GetChildOfName(_T("Peticion"))->GetValue());
	objEvts.SetVersion(node->GetChildOfName(_T("Version"))->GetValue());
	objEvts.SetOrigen(node->GetChildOfName(_T("OrigenInt"))->GetValue());

	CXMLNodeList* nodLst = node->GetChildsOfName(_T("Evento"));
	if(nodLst) {
		for(int i = 0; i < nodLst->GetCount(); i++) {
			CXMLNode* nod = nodLst->GetAt(i);
			CEventoMW* objEvt = new CEventoMW;
			objEvt->SetID(nod->GetAttributeOfName(_T("id"))->GetValue());
			objEvt->SetAsync(nod->GetAttributeOfName(_T("async"))->GetValue());
			objEvt->SetDumpResult(nod->GetAttributeOfName(_T("dumpResult"))->GetValue());
			objEvt->SetIDMensaje(nod->GetChildOfName(_T("IdMensaje"))->GetValue());
			CXMLNodeList* iufLst = nod->GetChildsOfName(_T("Iuff"));
			if(iufLst) {
				for(int i = 0; i < iufLst->GetCount(); i++) {
					CXMLNode* nod = iufLst->GetAt(i);
					CIuff* objIuff = new CIuff;
					objIuff->SetNombre(nod->GetAttributeOfName(_T("name"))->GetValue());
					objIuff->SetPersist(nod->GetAttributeOfName(_T("persist"))->GetValue());
					CXMLNode* nodVal = nod->GetChildOfName(_T("Value"));
					objIuff->SetLoadFrom(nodVal->GetValue());
					objIuff->SetTipo(nodVal->GetAttributeOfName(_T("type"))->GetValue());
					objEvt->AddIuff(objIuff);
				}
				delete iufLst;
			}
			CXMLNode* queued = nod->GetChildOfName(_T("WhenQueued"));
			if(queued) {
				objEvt->SetWhenQueued(queued->GetChildOfName(_T("Call"))->GetAttributeOfName(_T("function"))->GetValue());
			}
			objEvts.AddEvento(objEvt);
		}
		delete nodLst;
	}
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::MaquinaToXml(CString& strOut) {
	CString strTemp;

	strOut += _T("<Maquina>");
	strOut += _T("<Estados>");
	CMyMapIterador<CEstado*>* iterador = m_objWorkitem->GetMaquina()->GetEstados();
	if(iterador->begin()) {
		while(*iterador) {
			CEstado* est = (*iterador)++;
			EstadoToXml(est, strOut);
		}
	}
	strOut += _T("</Estados>");
	strOut += _T("<Transiciones>");
	if(iterador->begin()) {
		while(*iterador) {
			CEstado* est = (*iterador)++;
			const CArray<CTransicion*,CTransicion*>* array = est->GetTransiciones();
			for(int i = 0; i < array->GetSize(); i++) {
				CTransicion* tran = array->GetAt(i);
				TransicionToXml(tran, strOut);
			}
		}
	}
	delete iterador;
	strOut += _T("</Transiciones>");
	strOut += _T("</Maquina>");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::EstadoToXml(CEstado* oEst, CString& strOut) {
	CRect rc = oEst->GetRect();
	CString strTemp;
	strTemp.Format(_T("<Estado id=\"%s\" Left=\"%i\" Top=\"%i\" Width=\"%i\" Height=\"%i\">"),
		oEst->GetID(), rc.left, rc.top, rc.Width(), rc.Height()
		);
	strOut += strTemp;
	LPCTSTR strDesc = oEst->GetDescripcion();
	if(strDesc && lstrlen(strDesc)) {
		strOut += _T("<Descripcion>");
		strOut += oEst->GetDescripcion();
		strOut += _T("</Descripcion>");
	}
	if(oEst->IsInitial()) {
		strOut += _T("<Initial/>");
	}
	CMyMapIterador<CTimedAction*>* iterador = oEst->GetTimedActions();
	if(iterador->begin()) {
		strOut += _T("<TimedActions>");
		while(*iterador) {
			CTimedAction* objTA = (*iterador)++;
			// Esto es un asco, pero es para mantener la compatibilidad del xml
			if(objTA->TimeNeedsEval()) {
				strOut += _T("<TimedAction time=\"evaluate\">");
				strTemp.Format(_T("<CodeTime><![CDATA[%s]]></CodeTime>"), objTA->GetTime());
				strOut += strTemp;
				strTemp.Format(_T("<Code><![CDATA[%s]]></Code>"), objTA->GetCodigo());
				strOut += strTemp;
				strOut += _T("</TimedAction>");
			}
			else {
				strTemp.Format(_T("<TimedAction time=\"%s\"><![CDATA["), objTA->GetTime());
				strOut += strTemp;
				strOut += objTA->GetCodigo();
				strOut += _T("]]></TimedAction>");
			}
		}
		strOut += _T("</TimedActions>");
	}
	delete iterador;
	if(oEst->HayOnEntry()) {
		strOut += _T("<OnEntry><![CDATA[");
		strOut += oEst->GetOnEntry();
		strOut += _T("]]></OnEntry>");
	}
	if(oEst->HayOnExit()) {
		strOut += _T("<OnExit><![CDATA[");
		strOut += oEst->GetOnExit();
		strOut += _T("]]></OnExit>");
	}
	strOut += _T("</Estado>");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::TransicionToXml(CTransicion* oTran, CString& strOut) {
	CString strTemp;

	strTemp.Format(_T("<Transicion from=\"%s\" accion=\"%s\" to=\"%s\">"),
		oTran->GetOrigen()->GetID(), oTran->GetNombre(), oTran->GetDestino()->GetID()
		);
	strOut += strTemp;
	strOut += _T("<Descripcion>");
	strOut += oTran->GetDescripcion();
	strOut += _T("</Descripcion>");
	const CArray<CPoint,CPoint>* array = oTran->GetExtraPoints();
	if(array->GetSize()) {
		strOut += _T("<ExtraPoints>");
		for(int i = 0; i < array->GetSize(); i++) {
			CPoint pt = array->GetAt(i);
			strTemp.Format(_T("<Point x=\"%i\" y=\"%i\"/>"), pt.x, pt.y);
			strOut += strTemp;
		}
		strOut += _T("</ExtraPoints>");
	}
	if(oTran->HayCondicion()) {
		strOut += _T("<Condicion><![CDATA[");
		strOut += oTran->GetCondicion();
		strOut += _T("]]></Condicion>");
	}
	if(oTran->HayAccion()) {
		strTemp.Format(_T("<Accion type=\"%s\"><![CDATA["), oTran->GetTipoAccionStr());
		strOut += strTemp;
		strOut += oTran->GetAccion();
		strOut += _T("]]></Accion>");
	}
	strOut += _T("</Transicion>");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::MetodoToXml(CMetodo* oMet, CString& strOut) {
	CString strTemp;

	strTemp.Format(_T("<Metodo nombre=\"%s\"><![CDATA["), oMet->GetNombre());
	strOut += strTemp;
	strOut += oMet->GetCodigo();
	strOut += _T("]]></Metodo>");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
BOOL CWorkitemSerializer::EventosMWToXml(CString& strOut) {
	CEventosMW& objEvts = m_objWorkitem->GetEventosMW();
	CString strTemp;

	strOut += _T("<EventosMW>");
	strOut += _T("<Peticion>");
	strOut += objEvts.GetPeticion();
	strOut += _T("</Peticion>");
	strOut += _T("<Version>");
	strOut += objEvts.GetVersion();
	strOut += _T("</Version>");
	strOut += _T("<OrigenInt>");
	strOut += objEvts.GetOrigen();
	strOut += _T("</OrigenInt>");

	CMyMapIterador<CEventoMW*>* iterador = objEvts.GetEventos();
	if(iterador->begin()) {
		while(*iterador) {
			CEventoMW* oEvt = (*iterador)++;
			strTemp.Format(_T("<Evento id=\"%s\" async=\"%s\" dumpResult=\"%s\">"),
				oEvt->GetID(), oEvt->GetAsyncStr(), oEvt->GetDumpResultStr());
			strOut += strTemp;
			strOut += _T("<IdMensaje>");
			strOut += oEvt->GetIDMensaje();
			strOut += _T("</IdMensaje>");
			CMyMapIterador<CIuff*>* iter = oEvt->GetIuffs();
			if(iter->begin()) {
				while(*iter) {
					CIuff* oIuff = (*iter)++;
					strTemp.Format(_T("<Iuff name=\"%s\" persist=\"%s\">"),
						oIuff->GetNombre(), oIuff->GetPersistStr());
					strOut += strTemp;
					strTemp.Format(_T("<Value type=\"%s\">%s</Value>"),
						oIuff->GetTipoStr(), oIuff->GetLoadFrom());
					strOut += strTemp;
					strOut += _T("</Iuff>");
				}
			}
			delete iter;
			if(oEvt->HayWhenQueued()) {
				strOut += _T("<WhenQueued><Call function=\"");
				strOut += oEvt->GetWhenQueued();
				strOut += _T("\"/></WhenQueued>");
			}
			strOut += _T("</Evento>");
		}
	}
	delete iterador;
	strOut += _T("</EventosMW>");
	return TRUE;
}