#include "stdafx.h"
#include "compintdesigner.h"
#include "IuffsListCtrl.h"
#include "Domain\Iuff.h"

#define NUM_COLUMNAS 4

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_DYNAMIC(CIuffsListCtrl, CMyListCtrl)

CIuffsListCtrl::CIuffsListCtrl() {
	m_oldSortAscending = false;
	m_bCustomDraw = true;
}

CIuffsListCtrl::~CIuffsListCtrl() {

}

BEGIN_MESSAGE_MAP(CIuffsListCtrl, CMyListCtrl) 
   ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
   ON_WM_SIZE()
   ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()

//////////////////////////////////////////////////////////////////////
// 
bool CIuffsListCtrl::Init() {
	CRect rect;

	SetExtendedStyle(LVS_EX_FULLROWSELECT);
	ModifyStyle(NULL, LVS_SHOWSELALWAYS);

	GetClientRect(rect);
	int iSize = rect.Width() / NUM_COLUMNAS;

	InsertColumn(0, "Name", LVCFMT_CENTER, iSize);
	InsertColumn(1, "Type", LVCFMT_CENTER, iSize);
	InsertColumn(2, "Value", LVCFMT_CENTER, iSize);
	InsertColumn(3, "Persist", LVCFMT_CENTER, iSize);
	return true;
}

//////////////////////////////////////////////////////////////////////
// 
void CIuffsListCtrl::OnSize(UINT nType, int cx, int cy) {
	SetRedraw(FALSE);
	int iSize = cx / NUM_COLUMNAS;
	for(int i = 0; i < NUM_COLUMNAS; i++) {
		SetColumnWidth(i, iSize);
	}
	CMyListCtrl::OnSize(nType, cx, cy);
	SetRedraw(TRUE);
}

//////////////////////////////////////////////////////////////////////
// 
void CIuffsListCtrl::OnColumnClick(NMHDR *pNMHDR, LRESULT *pResult) {
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR; 

	if(!GetItemCount()) return;
	// Item is column clicked
	int sortItem = pNMListView->iSubItem;
	// Sort table
	m_oldSortAscending = !m_oldSortAscending;
	SetSortArrow(sortItem, m_oldSortAscending);
	SortItems(SortProc, sortItem + (m_oldSortAscending ? 0:100));
	this->Invalidate();
	*pResult = 0; 
}

//////////////////////////////////////////////////////////////////////
// 
int CIuffsListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort){ 
	CIuff* item1 = (CIuff*)lParam1;
	CIuff* item2 = (CIuff*)lParam2;
	if((item1 == NULL) || (item2 == NULL)) {
		return 0;
	}
	
	switch(lParamSort) {
		case 0:	// Sort ascendente
			return _tcsicmp(item1->GetNombre(), item2->GetNombre());
		break;

		case 100:	// Sort descendente
			return _tcsicmp(item2->GetNombre(), item1->GetNombre());
		break;

		case 1:	// Sort ascendente
			return _tcsicmp(item1->GetTipoStr(), item2->GetTipoStr());
		break;

		case 101:	// Sort descendente
			return _tcsicmp(item2->GetTipoStr(), item1->GetTipoStr());
		break;

		case 2:	// Sort ascendente
			return _tcsicmp(item1->GetLoadFrom(), item2->GetLoadFrom());
		break;

		case 102:	// Sort descendente
			return _tcsicmp(item2->GetLoadFrom(), item1->GetLoadFrom());
		break;

		case 3:	// Sort ascendente
			return _tcsicmp(item1->GetPersistStr(), item2->GetPersistStr());
		break;

		case 103:	// Sort descendente
			return _tcsicmp(item2->GetPersistStr(), item1->GetPersistStr());
		break;
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////
// 
int CIuffsListCtrl::GetSelected(CIuff** p_oEvt) {
	int nSel = GetNextItem(-1, LVNI_SELECTED);
	if(nSel == -1) {
		*p_oEvt = NULL;
		return nSel;
	}
	*p_oEvt = (CIuff*)GetItemData(nSel);
	return nSel;
}

//////////////////////////////////////////////////////////////////////
// 
int CIuffsListCtrl::Add(CIuff* oIuff, bool bSelect) {
	int iItem = InsertItem(0, oIuff->GetNombre());
	SetItemText(iItem, 1, oIuff->GetTipoStr());
	SetItemText(iItem, 2, oIuff->GetLoadFrom());
	SetItemText(iItem, 3, oIuff->GetPersistStr());
	SetItemData(iItem, (DWORD)oIuff);
	if(bSelect) {
		SetItemState(iItem, LVIS_SELECTED, LVIS_SELECTED);
	}
	return iItem;
}

//////////////////////////////////////////////////////////////////////
// 
void CIuffsListCtrl::OnContextMenu(CWnd* pWnd, CPoint point) {
	CMenu menu;
	// fix - on right click, we also want to change the current selection like the left click does 
	CPoint p = point;
	ScreenToClient(&p);
	int it = HitTest(p);
	if(it != -1) {
      SetSelectionMark(it);   // display selection mark correctly! 
	}
	// fix end 
	menu.CreatePopupMenu();
	menu.AppendMenu(MF_STRING, IDC_BUTTON_ADD, "Add");
	menu.AppendMenu(MF_STRING, IDC_BUTTON_EDIT, "Edit");
	menu.AppendMenu(MF_STRING, IDC_BUTTON_REMOVE, "Remove");

	menu.TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y, GetParent());
}