#if !defined(AFX_LUARUNTIME_H__5E316F4C_EE73_4483_B0F0_6619A9D7C653__INCLUDED_)
#define AFX_LUARUNTIME_H__5E316F4C_EE73_4483_B0F0_6619A9D7C653__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "LuaIncludes.h"

#define BEGIN_EVAL_FUNC_MAP(clase) \
	static clase::RegType* GetMethodsMap() {\
	static clase::RegType methods[] = {

#define END_EVAL_FUNC_MAP() \
	{ NULL, NULL } }; \
	return methods; };

#define EVAL_FUNC_ENTRY(name) \
{#name, name },

#define EVAL_FUNC_ENTRY2(name, func) \
{#name, func },

#define EVAL_FUNC(method) int method (lua_State* L)
#define EVAL_FUNCIMP(baseclass,funcnom)\
	int baseclass::funcnom(lua_State* L)

// El tipo TDest del template es la clase que implementa los mtodos que se van a llamar.
// El tipo TBase del template es la clase que deriva de CLuaRuntime

template <typename TDest, typename TBase>
class CLuaDoubleRuntime 
{
typedef struct { TDest *pT; } userdataType;

public:
	CLuaDoubleRuntime() {
		m_L = lua_open();
		if(m_L) {
			static_cast<TBase*>(this)->Initialize();
		}
	}
	virtual ~CLuaDoubleRuntime() {
		lua_close(m_L);
	}
	
	typedef int (TDest::*mfp)(lua_State *L);
	typedef struct RegType { const char *name; mfp mfunc; } RegType;

protected:
	// por defecto, abre todas las libreras
	void Initialize() {
		luaL_openlibs(m_L);
	};
	// por defecto, manda el string de error a la consola
	void OnError() {
		const char* strErr = lua_tostring(m_L, -1);
		OutputDebugString(strErr);
	};
	
public:
	inline operator lua_State* () const { return m_L; };

	inline int dostring(const char* s) {
		TBase* pT = static_cast<TBase*>(this);
		int iRet = luaL_loadstring(m_L, s);
		if(iRet == 0) {
   			iRet = lua_pcall(m_L, 0, LUA_MULTRET, 0);
			if(iRet != 0) {
				pT->OnError();
			}
		}
		else {
			pT->OnError();
		}
		return iRet;
	}

	//////////////////////////////////////////////////////////////////////////
	/// 
	void Register(TDest* pDest) {
		for(RegType* l = TBase::GetMethodsMap(); l->name; l++) {
			lua_pushstring(m_L, l->name);
			lua_pushlightuserdata(m_L, (void*)l);
			lua_pushlightuserdata(m_L, (void*)pDest);
			lua_pushcclosure(m_L, thunk, 2);
			lua_setglobal(m_L, l->name);
		}
		lua_settop(m_L, 0);
	}

	//////////////////////////////////////////////////////////////////////////
	/// 
	static int thunk(lua_State* L) {
		// stack has userdata, followed by method args
		//T *obj = check(L, 1);  // get 'self', or if you prefer, 'this'
		//lua_remove(L, 1);  // remove self so member function args start at index 1
		// get member function from upvalue
		TDest* obj = static_cast<TDest*>(lua_touserdata(L, lua_upvalueindex(2)));
		RegType* l = static_cast<RegType*>(lua_touserdata(L, lua_upvalueindex(1)));
		return (obj->*(l->mfunc))(L);  // call member function
	}

protected:
	lua_State* m_L;
};

template <typename TBase>
class CLuaRuntime 
{
typedef struct { TBase *pT; } userdataType;

public:
	CLuaRuntime() {
		m_L = lua_open();
		if(m_L) {
			static_cast<TBase*>(this)->Initialize();
		}
	}
	virtual ~CLuaRuntime() {
		lua_close(m_L);
	}
	
	typedef int (TBase::*mfp)(lua_State *L);
	typedef struct RegType { const char *name; mfp mfunc; } RegType;

protected:
	// por defecto, abre todas las libreras
	void Initialize() {
		luaL_openlibs(m_L);
	};
	// por defecto, manda el string de error a la consola
	void OnError() {
		const char* strErr = lua_tostring(m_L, -1);
		OutputDebugString(strErr);
	};
	
public:
	inline operator lua_State* () const { return m_L; };

	inline int dostring(const char* s) {
		int iRet = luaL_loadstring(m_L, s);
		if(iRet == 0) {
   			iRet = lua_pcall(m_L, 0, LUA_MULTRET, 0);
			if(iRet != 0) {
				TBase* pT = static_cast<TBase*>(this);
				pT->OnError();
			}
		}
		else {
			TBase* pT = static_cast<TBase*>(this);
			pT->OnError();
		}
		return iRet;
	}

	//////////////////////////////////////////////////////////////////////////
	/// 
	void Register(TBase* pBase) {
		for(RegType* l = TBase::GetMethodsMap(); l->name; l++) {
			lua_pushstring(m_L, l->name);
			lua_pushlightuserdata(m_L, (void*)l);
			lua_pushlightuserdata(m_L, (void*)pBase);
			//lua_pushlightuserdata(m_L, (void*)pDest);
			//lua_pushcclosure(m_L, thunk, 2);
			lua_pushcclosure(m_L, thunk, 2);
			lua_setglobal(m_L, l->name);
		}
		lua_settop(m_L, 0);
	}

	//////////////////////////////////////////////////////////////////////////
	/// 
	static int thunk(lua_State* L) {
		// get 'this' pointer from upvalue
		TBase* obj = static_cast<TBase*>(lua_touserdata(L, lua_upvalueindex(2)));
		// get member function from upvalue
		RegType* l = static_cast<RegType*>(lua_touserdata(L, lua_upvalueindex(1)));
		return (obj->*(l->mfunc))(L);  // call member function
	}

protected:
	lua_State* m_L;
};

#endif // !defined(AFX_LUARUNTIME_H__5E316F4C_EE73_4483_B0F0_6619A9D7C653__INCLUDED_)
